package com.sun.xml.internal.messaging.saaj.packaging.mime.internet;

import com.sun.xml.internal.messaging.saaj.packaging.mime.util.OutputUtil;
import java.io.OutputStream;
import javax.activation.DataSource;
import java.io.UnsupportedEncodingException;
import com.sun.xml.internal.messaging.saaj.util.FinalArrayList;
import java.util.Iterator;
import java.util.List;
import com.sun.xml.internal.org.jvnet.mimepull.Header;
import java.io.IOException;
import com.sun.xml.internal.messaging.saaj.packaging.mime.MessagingException;
import com.sun.xml.internal.messaging.saaj.util.ByteOutputStream;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import com.sun.xml.internal.org.jvnet.mimepull.MIMEPart;
import java.io.InputStream;
import javax.activation.DataHandler;

public final class MimeBodyPart
{
    public static final String ATTACHMENT = "attachment";
    public static final String INLINE = "inline";
    private static boolean setDefaultTextCharset;
    private DataHandler dh;
    private byte[] content;
    private int contentLength;
    private int start;
    private InputStream contentStream;
    private final InternetHeaders headers;
    private MimeMultipart parent;
    private MIMEPart mimePart;
    
    public MimeBodyPart() {
        this.start = 0;
        this.headers = new InternetHeaders();
    }
    
    public MimeBodyPart(InputStream is) throws MessagingException {
        this.start = 0;
        if (!(is instanceof ByteArrayInputStream) && !(is instanceof BufferedInputStream) && !(is instanceof SharedInputStream)) {
            is = new BufferedInputStream(is);
        }
        this.headers = new InternetHeaders(is);
        if ((is instanceof SharedInputStream)) {
            SharedInputStream sis = (SharedInputStream)is;
            this.contentStream = sis.newStream(sis.getPosition(), -1L);
        }
        else {
            try {
                ByteOutputStream bos = new ByteOutputStream();
                bos.write(is);
                this.content = bos.getBytes();
                this.contentLength = bos.getCount();
            }
            catch (IOException ioex) {
                throw new MessagingException("Error reading input stream", ioex);
            }
        }
    }
    
    public MimeBodyPart(InternetHeaders headers, byte[] content, int len) {
        this.start = 0;
        this.headers = headers;
        this.content = content;
        this.contentLength = len;
    }
    
    public MimeBodyPart(InternetHeaders headers, byte[] content, int start, int len) {
        this.start = 0;
        this.headers = headers;
        this.content = content;
        this.start = start;
        this.contentLength = len;
    }
    
    public MimeBodyPart(MIMEPart part) {
        this.start = 0;
        this.mimePart = part;
        this.headers = new InternetHeaders();
        List<? extends Header> hdrs = this.mimePart.getAllHeaders();
        for (Header hd : hdrs) {
            this.headers.addHeader(hd.getName(), hd.getValue());
        }
    }
    
    public MimeMultipart getParent() {
        return this.parent;
    }
    
    public void setParent(MimeMultipart parent) {
        this.parent = parent;
    }
    
    public int getSize() {
        if (this.mimePart != null) {
            try {
                return this.mimePart.read().available();
            }
            catch (IOException ex) {
                return -1;
            }
        }
        else {
            if (this.content != null) {
                return this.contentLength;
            }
            if (this.contentStream != null) {
                try {
                    int size = this.contentStream.available();
                    if (size > 0) {
                        return size;
                    }
                }
                catch (IOException ex2) {}
            }
            return -1;
        }
    }
    
    public int getLineCount() {
        return -1;
    }
    
    public String getContentType() {
        if (this.mimePart != null) {
            return this.mimePart.getContentType();
        }
        String s = this.getHeader("Content-Type", null);
        if (s == null) {
            s = "text/plain";
            return s;
        }
        return s;
    }
    
    public boolean isMimeType(String mimeType) {
        boolean result;
        try {
            ContentType ct = new ContentType(this.getContentType());
            result = ct.match(mimeType);
        }
        catch (ParseException ex) {
            result = this.getContentType().equalsIgnoreCase(mimeType);
        }
        return result;
    }
    
    public String getDisposition() throws MessagingException {
        String s = this.getHeader("Content-Disposition", null);
        if (s == null) {
            return null;
        }
        ContentDisposition cd = new ContentDisposition(s);
        return cd.getDisposition();
    }
    
    public void setDisposition(String disposition) throws MessagingException {
        if (disposition == null) {
            this.removeHeader("Content-Disposition");
        }
        else {
            String s = this.getHeader("Content-Disposition", null);
            if (s != null) {
                ContentDisposition cd = new ContentDisposition(s);
                cd.setDisposition(disposition);
                disposition = cd.toString();
            }
            this.setHeader("Content-Disposition", disposition);
        }
    }
    
    public String getEncoding() throws MessagingException {
        String s = this.getHeader("Content-Transfer-Encoding", null);
        if (s == null) {
            return null;
        }
        s = s.trim();
        if (s.equalsIgnoreCase("7bit") || s.equalsIgnoreCase("8bit") || s.equalsIgnoreCase("quoted-printable") || s.equalsIgnoreCase("base64")) {
            return s;
        }
        HeaderTokenizer h = new HeaderTokenizer(s, "()<>@,;:\\\"\t []/?=");
        int tkType;
        HeaderTokenizer.Token tk;
        do {
            tk = h.next();
            tkType = tk.getType();
            if (tkType == -4) {
                return s;
            }
            continue;
        } while (tkType != -1);
        return tk.getValue();
    }
    
    public String getContentID() {
        return this.getHeader("Content-ID", null);
    }
    
    public void setContentID(String cid) {
        if (cid == null) {
            this.removeHeader("Content-ID");
        }
        else {
            this.setHeader("Content-ID", cid);
        }
    }
    
    public String getContentMD5() {
        return this.getHeader("Content-MD5", null);
    }
    
    public void setContentMD5(String md5) {
        this.setHeader("Content-MD5", md5);
    }
    
    public String[] getContentLanguage() throws MessagingException {
        String s = this.getHeader("Content-Language", null);
        if (s == null) {
            return null;
        }
        HeaderTokenizer h = new HeaderTokenizer(s, "()<>@,;:\\\"\t []/?=");
        FinalArrayList v = new FinalArrayList();
        while (true) {
            HeaderTokenizer.Token tk = h.next();
            int tkType = tk.getType();
            if (tkType == -4) {
                break;
            }
            if (tkType != -1) {
                continue;
            }
            v.add(tk.getValue());
        }
        if (v.size() == 0) {
            return null;
        }
        return (String[])(String[])v.toArray(new String[v.size()]);
    }
    
    public void setContentLanguage(String[] languages) {
        StringBuffer sb = new StringBuffer(languages[0]);
        for (int i = 1; i < languages.length; ++i) {
            sb.append(',').append(languages[i]);
        }
        this.setHeader("Content-Language", sb.toString());
    }
    
    public String getDescription() {
        String rawvalue = this.getHeader("Content-Description", null);
        if (rawvalue == null) {
            return null;
        }
        try {
            return MimeUtility.decodeText(MimeUtility.unfold(rawvalue));
        }
        catch (UnsupportedEncodingException ex) {
            return rawvalue;
        }
    }
    
    public void setDescription(String description) throws MessagingException {
        this.setDescription(description, null);
    }
    
    public void setDescription(String description, String charset) throws MessagingException {
        if (description == null) {
            this.removeHeader("Content-Description");
            return;
        }
        try {
            this.setHeader("Content-Description", MimeUtility.fold(21, MimeUtility.encodeText(description, charset, null)));
        }
        catch (UnsupportedEncodingException uex) {
            throw new MessagingException("Encoding error", uex);
        }
        return;
    }
    
    public String getFileName() throws MessagingException {
        String filename = null;
        String s = this.getHeader("Content-Disposition", null);
        if (s != null) {
            ContentDisposition cd = new ContentDisposition(s);
            filename = cd.getParameter("filename");
        }
        if (filename == null) {
            s = this.getHeader("Content-Type", null);
            if (s != null) {
                try {
                    ContentType ct = new ContentType(s);
                    filename = ct.getParameter("name");
                }
                catch (ParseException ex) {}
            }
        }
        return filename;
    }
    
    public void setFileName(String filename) throws MessagingException {
        String s = this.getHeader("Content-Disposition", null);
        ContentDisposition cd = new ContentDisposition((s == null) ? "attachment" : s);
        cd.setParameter("filename", filename);
        this.setHeader("Content-Disposition", cd.toString());
        s = this.getHeader("Content-Type", null);
        if (s != null) {
            try {
                ContentType cType = new ContentType(s);
                cType.setParameter("name", filename);
                this.setHeader("Content-Type", cType.toString());
            }
            catch (ParseException ex) {}
        }
    }
    
    public InputStream getInputStream() throws IOException {
        return this.getDataHandler().getInputStream();
    }
    
    InputStream getContentStream() throws MessagingException {
        if (this.mimePart != null) {
            return this.mimePart.read();
        }
        if (this.contentStream != null) {
            return ((SharedInputStream)this.contentStream).newStream(0L, -1L);
        }
        if (this.content != null) {
            return new ByteArrayInputStream(this.content, this.start, this.contentLength);
        }
        throw new MessagingException("No content");
    }
    
    public InputStream getRawInputStream() throws MessagingException {
        return this.getContentStream();
    }
    
    public DataHandler getDataHandler() {
        if (this.mimePart != null) {
            return new DataHandler(new DataSource() {
                public InputStream getInputStream() throws IOException {
                    return MimeBodyPart.this.mimePart.read();
                }
                
                public OutputStream getOutputStream() throws IOException {
                    throw new UnsupportedOperationException("getOutputStream cannot be supported : You have enabled LazyAttachments Option");
                }
                
                public String getContentType() {
                    return MimeBodyPart.this.mimePart.getContentType();
                }
                
                public String getName() {
                    return "MIMEPart Wrapped DataSource";
                }
            });
        }
        if (this.dh == null) {
            this.dh = new DataHandler(new MimePartDataSource(this));
        }
        return this.dh;
    }
    
    public Object getContent() throws IOException {
        return this.getDataHandler().getContent();
    }
    
    public void setDataHandler(DataHandler dh) {
        if (this.mimePart != null) {
            this.mimePart = null;
        }
        this.dh = dh;
        this.content = null;
        this.contentStream = null;
        this.removeHeader("Content-Type");
        this.removeHeader("Content-Transfer-Encoding");
    }
    
    public void setContent(Object o, String type) {
        if (this.mimePart != null) {
            this.mimePart = null;
        }
        if ((o instanceof MimeMultipart)) {
            this.setContent((MimeMultipart)o);
        }
        else {
            this.setDataHandler(new DataHandler(o, type));
        }
    }
    
    public void setText(String text) {
        this.setText(text, null);
    }
    
    public void setText(String text, String charset) {
        if (charset == null) {
            if (MimeUtility.checkAscii(text) != 1) {
                charset = MimeUtility.getDefaultMIMECharset();
            }
            else {
                charset = "us-ascii";
            }
        }
        this.setContent(text, "text/plain; charset=" + MimeUtility.quote(charset, "()<>@,;:\\\"\t []/?="));
    }
    
    public void setContent(MimeMultipart mp) {
        if (this.mimePart != null) {
            this.mimePart = null;
        }
        this.setDataHandler(new DataHandler(mp, mp.getContentType().toString()));
        mp.setParent(this);
    }
    
    public void writeTo(OutputStream os) throws IOException, MessagingException {
        List<Object> hdrLines = this.headers.getAllHeaderLines();
        for (int sz = hdrLines.size(), i = 0; i < sz; ++i) {
            OutputUtil.writeln((String)hdrLines.get(i), os);
        }
        OutputUtil.writeln(os);
        if (this.contentStream != null) {
            ((SharedInputStream)this.contentStream).writeTo(0L, -1L, os);
        }
        else if (this.content != null) {
            os.write(this.content, this.start, this.contentLength);
        }
        else if (this.dh != null) {
            OutputStream wos = MimeUtility.encode(os, this.getEncoding());
            this.getDataHandler().writeTo(wos);
            if (os != wos) {
                wos.flush();
            }
        }
        else if (this.mimePart != null) {
            OutputStream wos = MimeUtility.encode(os, this.getEncoding());
            this.getDataHandler().writeTo(wos);
            if (os != wos) {
                wos.flush();
            }
        }
        else {
            throw new MessagingException("no content");
        }
    }
    
    public String[] getHeader(String name) {
        return this.headers.getHeader(name);
    }
    
    public String getHeader(String name, String delimiter) {
        return this.headers.getHeader(name, delimiter);
    }
    
    public void setHeader(String name, String value) {
        this.headers.setHeader(name, value);
    }
    
    public void addHeader(String name, String value) {
        this.headers.addHeader(name, value);
    }
    
    public void removeHeader(String name) {
        this.headers.removeHeader(name);
    }
    
    public FinalArrayList getAllHeaders() {
        return this.headers.getAllHeaders();
    }
    
    public void addHeaderLine(String line) {
        this.headers.addHeaderLine(line);
    }
    
    protected void updateHeaders() throws MessagingException {
        DataHandler dh = this.getDataHandler();
        if (dh == null) {
            return;
        }
        try {
            String type = dh.getContentType();
            boolean composite = false;
            boolean needCTHeader = this.getHeader("Content-Type") == null;
            ContentType cType = new ContentType(type);
            if (cType.match("multipart/*")) {
                composite = true;
                Object o = dh.getContent();
                ((MimeMultipart)o).updateHeaders();
            }
            else if (cType.match("message/rfc822")) {
                composite = true;
            }
            if (!composite) {
                if (this.getHeader("Content-Transfer-Encoding") == null) {
                    this.setEncoding(MimeUtility.getEncoding(dh));
                }
                if (needCTHeader && MimeBodyPart.setDefaultTextCharset && cType.match("text/*") && cType.getParameter("charset") == null) {
                    String enc = this.getEncoding();
                    String charset;
                    if (enc != null && enc.equalsIgnoreCase("7bit")) {
                        charset = "us-ascii";
                    }
                    else {
                        charset = MimeUtility.getDefaultMIMECharset();
                    }
                    cType.setParameter("charset", charset);
                    type = cType.toString();
                }
            }
            if (needCTHeader) {
                String s = this.getHeader("Content-Disposition", null);
                if (s != null) {
                    ContentDisposition cd = new ContentDisposition(s);
                    String filename = cd.getParameter("filename");
                    if (filename != null) {
                        cType.setParameter("name", filename);
                        type = cType.toString();
                    }
                }
                this.setHeader("Content-Type", type);
            }
        }
        catch (IOException ex) {
            throw new MessagingException("IOException updating headers", ex);
        }
        return;
    }
    
    private void setEncoding(String encoding) {
        this.setHeader("Content-Transfer-Encoding", encoding);
    }
    
    static {
        MimeBodyPart.setDefaultTextCharset = true;
        try {
            String s = System.getProperty("mail.mime.setdefaulttextcharset");
            MimeBodyPart.setDefaultTextCharset = (s == null || !s.equalsIgnoreCase("false"));
        }
        catch (SecurityException ex) {}
    }
}
